clear all

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
training.type=0;  % 0- binary RTRBM, 1- Gaussian RTRBM, 2- Spike and Slab RTRBM
training.nCD=25;  % number of contrastive-divergence iterations
training.num_of_nodes=25;  % number of nodes in the graph
training.NVisible=ones(training.num_of_nodes,1)*900/25;   % a vector wth the number of observations in each node
training.NHidden=ones(training.num_of_nodes,1)*150;   % a vector with the number of hidden units in each node
training.number_of_iterations=100000;%400000; % using more iterations tends to improve the performance but it is slower to train

training.learning_rate=0.005;
training.momentum=0.9;
training.l1_Theta_reg=0.001;   % SRTRBM regularization 
training.flag_learn_mask=0;    % 0- don't learn mask, 1- learn mask
training.flag_causal=0;        % 0- learn undirected graph, 1- learn directed graph
training.flag_share_lambda_parm=0; % 0- a single ssRBM lambda parameter for all groups, 1- share the ssRBM parameter lambda only within each group
training.alpha_parm=1.0;       % alpha hyper-parameter for ssRBM
training.lambda_parm_init=10.5; % initialization for lambda for ssRBM
training.delta_parm=8.0;  % delta hyper-parameter for the SRTRBM step-function
training.kappa=-3; % kappa hyper-parameter for the SRTRBM step-function
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
training.data_file='data/BouncingBalls/bouncing_balls_training_data.dat';
training.parameters_file='results/BouncingBalls_RTRBM_parameters.dat';
tmp=0;
if ~(exist(training.parameters_file)==2)
    save(training.parameters_file,'tmp');
end
if (exist(training.data_file)==2) && (exist(training.parameters_file)==2)
    [vct]=create_training_file(training);
    parms=[];
    for i=1:length(vct)
        parms=[parms,num2str(vct(i))];
        parms=[parms,' '];
    end
    parms=[parms,training.data_file,' ',training.parameters_file];
    system(['./train_RTRBM_cuda ',parms]);
else
    disp('The data file does not exist, or that there is an issue with the provided parameters file name');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
testing=training;
testing.data_file='data/BouncingBalls/bouncing_balls_testing_data.dat';
testing.parameters_file=training.parameters_file;
testing.nCD=5;
testing.number_of_iterations=0; % use 0 in order to average the results over all the testing sequences, otherwise every testing sequence is averaged using testing.number_of_iterations trials
if (exist(testing.data_file)==2) && (exist(testing.parameters_file)==2)
    [vct]=create_training_file(testing);
    parms=[];
    for i=1:length(vct)
        parms=[parms,num2str(vct(i))];
        parms=[parms,' '];
    end
    parms=[parms,testing.data_file,' ',testing.parameters_file];
    system(['./test_RTRBM_cuda ',parms]);
else
    disp('The data or parameters file does not exist');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (testing.type==0)
    testing.nCD=5;
    testing.number_of_iterations=10; % The index of the video in the testing set, whose first half is used as initialization
    parms=[];
    for i=1:length(vct)
        parms=[parms,num2str(vct(i))];
        parms=[parms,' '];
    end
    parms=[parms,testing.data_file,' ',testing.parameters_file];
    system(['./generate_synthetic_video_cuda ',parms]);
    addpath('data/BouncingBalls');
    NBlocksSide=5;
    BlockSideLen=6;
    T=create_obs_transform_matrix(NBlocksSide,BlockSideLen);
    video=load('results/synthetic_video.dat');
    num_rows=video(1);
    num_cols=video(2);
    synthetic_video=reshape(T'*reshape(video(3:end),[num_rows,num_cols]),[sqrt(num_rows),sqrt(num_rows),num_cols]);
    video=load('results/original_video.dat');
    num_rows=video(1);
    num_cols=video(2);
    original_video=reshape(T'*reshape(video(3:end),[num_rows,num_cols]),[sqrt(num_rows),sqrt(num_rows),num_cols]);
    video=0.5*ones(size(synthetic_video,1)+size(original_video,1)+6,size(synthetic_video,2)+4,size(synthetic_video,3));
    video(3:2+size(original_video,1),3:2+size(original_video,2),:)=original_video;
    video(5+size(original_video,1):4+size(original_video,1)+size(synthetic_video,1),3:2+size(original_video,2),:)=synthetic_video;
    implay(video);
end