[SRTRBM: Structured Recurrent Temporal Restricted Boltzmann Machines](http://www-personal.umich.edu/~rmittelm/)

Created by [Roni Mittelman](http://www-personal.umich.edu/~rmittelm/), University of Michigan EECS department.

## Introduction

This code implement the structured recurrent temporal restricted Boltzmann machine, from the 2014 ICML paper with the same title. It is assumed that the ArrayFire package (http://www.accelereyes.com/arrayfire/c/) is installed. Note that the code includes some bug fixes and different hyper-parameters settings compared to those that were used to generate the results in the paper, and therefore the results for all the spike-and-slab-based algorithms are slightly better than those in the paper.

## License

Please note that this code should be used at your own risk. There is no implied guarantee that it will
not do anything stupid. Permission is granted to use and modify the code.

## Citing SRTRBM

Please cite our ICML paper in your publications if it helps your research:

    @inproceedings{Mittelman14srtrbm,
      Author = {R. Mittelman, B. Kuipers, S. Savarese, and H. Lee},
      Title = {Structured recurrent temporal restricted Boltzmann machines},
      booktitle={ICML},
      Year  = {2014}
    }


## Installation

In the root SRTRBM directory. First, build the code using the following (if it does not work then just run make for any of the examples in the arrayfire examples directory to get the correct format)

1) g++ -m64 -Wall -Werror -I/usr/local/arrayfire/include -pthread -O2 -DNDEBUG -I/usr/local/cuda/include -lrt -Wl,--no-as-needed -L/usr/local/arrayfire/lib64 -lpthread -lstdc++ -lm -Wl,-rpath,/usr/local/arrayfire/lib64,-rpath,/usr/local/arrayfire/lib64 -lafcu -L/usr/local/cuda/lib64 -lcuda -lcudart -lcurand -lcusparse -Wl,-rpath,/usr/local/cuda/lib64 util.h util.cpp ContrastiveDivergence.h ContrastiveDivergence.cpp ssRTRBMrelease.h ssRTRBMrelease.cpp RTRBMrelease.h RTRBMrelease.cpp train_RTRBM.cpp -o train_RTRBM_cuda

2) g++ -m64 -Wall -Werror -I/usr/local/arrayfire/include -pthread -O2 -DNDEBUG -I/usr/local/cuda/include -lrt -Wl,--no-as-needed -L/usr/local/arrayfire/lib64 -lpthread -lstdc++ -lm -Wl,-rpath,/usr/local/arrayfire/lib64,-rpath,/usr/local/arrayfire/lib64 -lafcu -L/usr/local/cuda/lib64 -lcuda -lcudart -lcurand -lcusparse -Wl,-rpath,/usr/local/cuda/lib64 util.h util.cpp ContrastiveDivergence.h ContrastiveDivergence.cpp ssRTRBMrelease.h ssRTRBMrelease.cpp RTRBMrelease.h RTRBMrelease.cpp test_RTRBM.cpp -o test_RTRBM_cuda

3) g++ -m64 -Wall -Werror -I/usr/local/arrayfire/include -pthread -O2 -DNDEBUG -I/usr/local/cuda/include -lrt -Wl,--no-as-needed -L/usr/local/arrayfire/lib64 -lpthread -lstdc++ -lm -Wl,-rpath,/usr/local/arrayfire/lib64,-rpath,/usr/local/arrayfire/lib64 -lafcu -L/usr/local/cuda/lib64 -lcuda -lcudart -lcurand -lcusparse -Wl,-rpath,/usr/local/cuda/lib64 util.h util.cpp ContrastiveDivergence.h ContrastiveDivergence.cpp ssRTRBMrelease.h ssRTRBMrelease.cpp RTRBMrelease.h RTRBMrelease.cpp generate_synthetic_video.cpp -o generate_synthetic_video_cuda

Next, open matlab and run:
1) mex readParms_RTRBM.cpp
2) mex readParms_ssRTRBM.cpp
3) mex MatFile2DatFile.cpp

## Running the examples

- Bouncing balls example:

First, in order to generate the bouncing balls data, download the RTRBM code by Ilya Sutskever from http://www.cs.utoronto.ca/~ilya/code/2008/RTRBM.tar
and copy the files bouncing_balls.py and pylab.py to the directory SRTRBM/data/BouncingBalls
Next chnage the directory to data/BouncingBalls and run
1) python generate_BouncingBalls_data.py
2) matlab reorder_BouncingBalls_videos.m

You can now run the code for RTRBM and SRTRBM using the matlab wrapper scripts: BouncingBalls_RTRBM.m and BouncingBalls_SRTRBM.m respectively

- Motion capture example
 
First, change the directory to SRTRBM/data/MOCAP and save the data to the right format using:
matlab partition_MOCAP_dataset.m

Next, change the directory to SRTRBM and run any of the matlab wrapper scripts:
MOCAP_ssRBM_RTRBM.m for the spike-and-slab-based RTRBM
MOCAP_ssRBM_SRTRBM.m for the spike-and-slab-based SRTRBM 
MOCAP_ssRBM_GRTRBM.m for the spike-and-slab-based Gaussian RTRBM
MOCAP_ssRBM_RTRBM_OneShotLearning.m for the spike-and-slab-based RTRBM, using the smaller set for training, and the larger for testing 
MOCAP_ssRBM_SRTRBM_OneShotLearning.m for the spike-and-slab-based SRTRBM, using the smaller set for training, and the larger for testing 

In order to plot the graph you will need to have the python igraph library (http://igraph.org/python/) installed. To plot the graph first run MOCAP_ssRBM_SRTRBM.m in order to save the Mask to the file results/MOCAP/MOCAP_MASK.mat and then run:
python plot_MOCAPgraph_SRTRBM.py 

- USHCN dataset

First, change the directory to SRTRBM/data/weather and save the data to the right format using 
matlab partition_weather_dataset.m

Next, change the directory to SRTRBM and run any of the matlab wrapper scripts:
weather_ssRBM_RTRBM.m for the spike-and-slab-based RTRBM
weather_ssRBM_SRTRBM.m for the spike-and-slab-based SRTRBM

In order to learn the graph structure using SRTRBM we used both the training and testing sets. To train the model and show the graph run the matlab script
weather_ssRBM_SRTRBM_1and2.m

 



