

//
//  RBM.cpp
//  
//
//  Created by Roni Mittelman on 7/31/13.
//
//

#include "ssRTRBMrelease.h"
#include <algorithm>

using namespace af;

// use masks for W, Wprime, and phi

ssRTRBM::ssRTRBM(ContrastiveDivergence *CD0, int MiniBatchLen0, float learning_rate0, int NHidden0, int NVisible0, float momentum0, float alpha_parm0, float lambda_parm_init){
    CD=CD0;
    learning_rate=constant((float)learning_rate0,1);
    MiniBatchLen=1;
    NHidden=NHidden0;
    NVisible=NVisible0;
    W=randn(NVisible,NHidden)*0.1;
    Wprime=randn(NHidden,NHidden)*0.005;
    phi=abs(randn(NVisible,NHidden))*0.005;
    mu=randn(NHidden,1)*0.005;
    b=randu(NHidden)*(-4);
    binit=randu(NHidden)*(-4);
    c=randn(NVisible)*0.005; 
    alpha_parm=constant(alpha_parm0,1);
    lambda_parm=constant(lambda_parm_init,1);

    CD->updateParms(W, b, c,phi,mu, alpha_parm,lambda_parm);
    momentum=momentum0;
    Winc=constant(0,NVisible,NHidden);
    Wprime_inc=constant(0,NHidden,NHidden);
    phi_inc=constant(0,NVisible,NHidden);
    mu_inc=constant(0,NHidden,1);
    binc=constant(0,NHidden);
    binit_inc=constant(0,NHidden);
    cinc=constant(0,NVisible);
    WMask=constant(1,1);
    WprimeMask=constant(1,1);
    l1_Theta_reg=constant(0,1);
    flag_learn_mask=0;
    flag_causal=0;  //0: non-causal, 1: causal
    lambda_parm_inc=constant(0,1);
    ////////////////////
    num_of_nodes=NVisible;
    std::valarray<int> NVisible0_vct(1,NVisible);
    OnesMatV=createOnesMatrix(NVisible0_vct);
    flag_share_lambda_parm=0;
}

ssRTRBM::ssRTRBM(ContrastiveDivergence *CD0, int MiniBatchLen0, float learning_rate0, std::valarray<int> NHidden0_vct, std::valarray<int>  NVisible0_vct, float momentum0, float l1_Theta_reg0, int flag_learn_mask0, int flag_causal0, float alpha_parm0, float lambda_parm_init, int flag_share_lambda_parm0, float delta0, float kappa0){
    CD=CD0;
    flag_share_lambda_parm=flag_share_lambda_parm0;
    NHidden=(int)NHidden0_vct.sum();
    NVisible=(int)NVisible0_vct.sum();
    learning_rate=constant((float)learning_rate0,1);
    MiniBatchLen=1;
    int i;
    num_of_nodes=(int)NHidden0_vct.size();
    num_of_nodes=(int)NHidden0_vct.size();
    OnesMatV=createOnesMatrix(NVisible0_vct);
    OnesMatH=createOnesMatrix(NHidden0_vct);

    array Matu=constant(0,num_of_nodes,num_of_nodes);
    array Matl=constant(0,num_of_nodes,num_of_nodes);
    int k=0;
    for (i=0;i<num_of_nodes-1;i++){
        Matu(i,seq(i+1,end))=1;
        Matl(seq(i+1,end),i)=1;
        k+=num_of_nodes-1-i;
    }
    upper_diag_indices=flat(Matu);
    lower_diag_indices=flat(Matl);
    ///////////////////////////
    flag_causal=flag_causal0;  //0: non-causal, 1: causal
    step_function_factor=delta0;
    kappa=kappa0;
    b=randu(NHidden)*(-4);
    binit=randu(NHidden)*(-4);
    c=randn(NVisible)*0.005;
    alpha_parm=constant(alpha_parm0,1);
    lambda_parm=constant(lambda_parm_init,num_of_nodes);
    if (flag_causal==0){
        ThetaMask=randn(num_of_nodes*(num_of_nodes-1)/2)*0.005;
        ThetaMask_inc=constant(0,num_of_nodes*(num_of_nodes-1)/2);
    }
    else{
        ThetaMask=randn(num_of_nodes*(num_of_nodes-1))*0.005;
        ThetaMask_inc=constant(0,num_of_nodes*(num_of_nodes-1));
    }

    W=randn(NVisible,NHidden)*0.1;
    Wprime=randn(NHidden,NHidden)*0.005;
    phi=abs(randn(NVisible,NHidden))*0.005;
    mu=randn(NHidden,1)*0.005;
    if (flag_learn_mask0==1){
        WMask=Theta2MaskMat(ThetaMask, OnesMatV, OnesMatH);
        WprimeMask=Theta2MaskMat(ThetaMask, OnesMatH, OnesMatH);
    }
    else{
        WMask=constant(1,1);
        WprimeMask=constant(1,1);
    }
    momentum=momentum0;
    Winc=constant(0,NVisible,NHidden);
    Wprime_inc=constant(0,NHidden,NHidden);
    phi_inc=constant(0,NVisible,NHidden);
    mu_inc=constant(0,NHidden,1);
    binc=constant(0,NHidden);
    binit_inc=constant(0,NHidden);
    cinc=constant(0,NVisible);
    l1_Theta_reg=constant((float)l1_Theta_reg0,1);
    flag_learn_mask=flag_learn_mask0;
    lambda_parm_inc=constant(0,num_of_nodes);
    if ((flag_learn_mask==0) || (flag_share_lambda_parm==0)){
        lambda_parm=constant(lambda_parm_init,1);
        lambda_parm_inc=constant(0,1);
    }
    CD->updateParms(W, b, c,phi,mu, alpha_parm,lambda_parm);
}



ssRTRBM::ssRTRBM(ContrastiveDivergence *CD0){
    CD=CD0;
}

void ssRTRBM::update_l1_Theta_reg(float l1_Theta_reg0){
    l1_Theta_reg=constant(l1_Theta_reg0,1);
}

array ssRTRBM::createOnesMatrix(std::valarray<int> vct){
    array mat=constant(0,(int)vct.sum(),(int)vct.size());
    int k=0;
    int j;
    for (int i=0;i<(int)mat.dims()[1];i++){
        for (j=0;j<vct[i];j++){
            mat(k++,i)=1;
        }
    }
    return mat;
}

array ssRTRBM::sigma_function(array x){
    return (1.0/(1.0+exp(-x*step_function_factor)));
}

array ssRTRBM::compute_thetaProd(array MatA, array MatB, array W0, array OnesL, array OnesR){
    array vect=constant(0,num_of_nodes*num_of_nodes);
    array tmp;
    vect=matmul(matmul(OnesL.T(),matmul(MatA,MatB.T())*W0),OnesR);
    array vct;
    if (flag_causal==0){
        vct=vect.T()(where(lower_diag_indices>0))+vect(where(lower_diag_indices>0));
    }
    else{
        vct=join(0,vect(where(upper_diag_indices>0)),vect(where(lower_diag_indices>0)));
    }
    return vct;
}

array ssRTRBM::Theta2MaskMat(array theta_vct, array OnesL, array OnesR){
    array Mat=identity(num_of_nodes,num_of_nodes);
    Mat=flat(Mat);
    /////////////
    if (flag_causal==0){
        array tmpMat=identity(num_of_nodes,num_of_nodes);
        tmpMat(where(lower_diag_indices>0))=sigma_function(theta_vct);
        Mat=tmpMat+tmpMat.T()-identity(num_of_nodes,num_of_nodes);
    }
    else{
        Mat(where(upper_diag_indices>0))=sigma_function(theta_vct(seq(num_of_nodes*(num_of_nodes-1)/2)));
        Mat(where(lower_diag_indices>0))=sigma_function(theta_vct(seq(num_of_nodes*(num_of_nodes-1)/2,end)));
    }
    Mat=array(Mat,num_of_nodes,num_of_nodes);
    return matmul(matmul(OnesL,Mat),OnesR.T());
}

array ssRTRBM::Theta2Mat(array theta_vct){
    array Mat=identity(num_of_nodes,num_of_nodes);
    Mat=flat(Mat);
    if (flag_causal==0){
        array tmpMat=identity(num_of_nodes,num_of_nodes);
        tmpMat(where(lower_diag_indices>0))=sigma_function(theta_vct);
        Mat=tmpMat+tmpMat.T()-identity(num_of_nodes,num_of_nodes);
    }
    else{
        Mat(where(upper_diag_indices>0))=sigma_function(theta_vct(seq(num_of_nodes*(num_of_nodes-1)/2)));
        Mat(where(lower_diag_indices>0))=sigma_function(theta_vct(seq(num_of_nodes*(num_of_nodes-1)/2,end)));
    }
    Mat=array(Mat,num_of_nodes,num_of_nodes);
    return Mat;
}

void ssRTRBM::set_momentum(float momentum0){
    momentum=momentum0;
}

int ssRTRBM::getNHidden(){
    return NHidden;
}

void ssRTRBM::update_batch(array vbatch){
    int t;

    const int T=(int)vbatch.dims()[1];
    array Htmp=constant(0,NHidden,T);
    array tmp=constant(0,NHidden,T);
    array WM=WMask*W;
    array WprimeM=WprimeMask*Wprime;
    array phiM=phi*WMask;
    array tm0=matmul(WM.T(),vbatch(span,0));
    tmp(span,0)=0.5/alpha_parm*tm0*tm0+matmul(WM.T(),vbatch(span,0))*mu-0.5*matmul(phiM.T(),vbatch(span,0)*vbatch(span,0))+binit;
    Htmp(span,0)=1/(exp(-tmp(span,0))+1);
    for (t=1;t<T;t++){
        tm0=matmul(WM.T(),vbatch(span,t));
        tmp(span,t)=0.5/alpha_parm*tm0*tm0+matmul(WM.T(),vbatch(span,t))*mu-0.5*matmul(phiM.T(),vbatch(span,t)*vbatch(span,t))+b+matmul(WprimeM,Htmp(span,t-1));
        Htmp(span,t)=1/(exp(-tmp(span,t))+1);
    }
    if ((flag_learn_mask==1) && (flag_share_lambda_parm==1)){
        CD->updateParms(WM,join(1,binit,tile(b,1,T-1)+matmul(WprimeM,Htmp(span,seq(T-1)))),c, phiM, mu,alpha_parm,matmul(OnesMatV,lambda_parm));
    }
    else{
        CD->updateParms(WM,join(1,binit,tile(b,1,T-1)+matmul(WprimeM,Htmp(span,seq(T-1)))),c, phiM, mu,alpha_parm,lambda_parm);
    }
    CD->sampleCD(vbatch.T());

    ///////////////////////////////
    array h_a=CD->get_h_a();
    array h_b=CD->get_h_b();
    array s_a=CD->get_s_a();
    array s_b=CD->get_s_b();
    array v_b=CD->get_v_b();
    array Eh_a=CD->get_Eh_a();
    array Eh_b=CD->get_Eh_b();
    array Ev_b=CD->get_Ev_b();
    array dLdRt=constant(0,NHidden,T);
    array rtp1;
    for (t=T-2;t>=0;t--){
        rtp1=Htmp(span,t+1);
        dLdRt(span,t)=matmul(WprimeM.T(),rtp1*(1-rtp1)*dLdRt(span,t+1)+Eh_a(span,t+1)-Eh_b(span,t+1));
    }
    array grad_lambda;
    if ((flag_learn_mask==1) && (flag_share_lambda_parm==1)){
        grad_lambda=0.5*(-sum(matmul(OnesMatV.T(),vbatch*vbatch),1)+sum(matmul(OnesMatV.T(),v_b.T()*v_b.T()),1));
    }
    else{
        grad_lambda=sum(0.5*(-sum(vbatch*vbatch,1)+sum(v_b.T()*v_b.T(),1)));
    }
    
    array dpdW=matmul(vbatch,(Eh_a*s_a).T())-matmul(v_b.T(),(Eh_b*s_b).T());
    array gradThetaMask, gradThetaMask2;
    if (flag_learn_mask==1){
        gradThetaMask=compute_thetaProd(vbatch, Eh_a, W, OnesMatV, OnesMatH)-compute_thetaProd(v_b.T(), Eh_b, W, OnesMatV, OnesMatH)-0.5*(compute_thetaProd(vbatch*vbatch, Eh_a, phi, OnesMatV, OnesMatH)-compute_thetaProd(v_b.T()*v_b.T(), Eh_b, phi, OnesMatV, OnesMatH));;
        gradThetaMask+=compute_thetaProd(Eh_a(span,seq(1,T-1)),Htmp(span,seq(T-1)), Wprime,OnesMatH, OnesMatH)-compute_thetaProd(Eh_b(span,seq(1,T-1)),Htmp(span,seq(T-1)), Wprime,OnesMatH, OnesMatH);
    }
    array gradWprime=matmul(dLdRt(span,seq(1,T-1))*Htmp(span,seq(1,T-1))*(1-Htmp(span,seq(1,T-1))),Htmp(span,seq(T-1)).T());
    array gradB=sum(dLdRt(span,seq(1,T-1))*Htmp(span,seq(1,T-1))*(1-Htmp(span,seq(1,T-1))),1);
    array gradW=dpdW+matmul(dLdRt*Htmp*(1-Htmp)*matmul(WM.T(),vbatch),vbatch.T()).T()/alpha_parm+matmul(dLdRt*Htmp*(1-Htmp)*tile(mu,1,T),vbatch.T()).T();
    array grad_phi=0.5*(-matmul(vbatch*vbatch,Eh_a.T())+matmul((v_b*v_b).T(),Eh_b.T()));
    grad_phi-=0.5*matmul(vbatch*vbatch,(dLdRt*Htmp*(1-Htmp)).T());
    array grad_mu=alpha_parm*(sum(s_a*Eh_a,1)-sum(s_b*Eh_b,1)-mu*(sum(Eh_a,1)-sum(Eh_b,1)));
    grad_mu+=sum(matmul(vbatch.T(),WM).T()*dLdRt*Htmp*(1-Htmp),1);

    if (flag_learn_mask==1){
        gradThetaMask+=compute_thetaProd(vbatch,dLdRt*Htmp*(1-Htmp)*matmul(WM.T(),vbatch),W, OnesMatV, OnesMatH)/alpha_parm+compute_thetaProd(vbatch,dLdRt*Htmp*(1-Htmp)*tile(mu,1,T),W, OnesMatV, OnesMatH)-0.5*compute_thetaProd(vbatch*vbatch,dLdRt*Htmp*(1-Htmp),phi, OnesMatV, OnesMatH);
        gradThetaMask+=compute_thetaProd(dLdRt(span,seq(1,T-1))*Htmp(span,seq(1,T-1))*(1-Htmp(span,seq(1,T-1))),Htmp(span,seq(T-1)), Wprime,OnesMatH, OnesMatH);
    }

    gradWprime+=matmul(Eh_a(span,seq(1,T-1))-Eh_b(span,seq(1,T-1)),Htmp(span,seq(T-1)).T());
    gradB+=sum(Eh_a(span,seq(1,T-1))-Eh_b(span,seq(1,T-1)),1);
    array gradBinit=Eh_a(span,0)-Eh_b(span,0)+dLdRt(span,0)*Htmp(span,0)*(1-Htmp(span,0));

    gradW/=T;
    gradWprime/=T;
    gradB/=T;
    grad_phi/=T;
    grad_mu/=T;
    gradBinit/=T;
    if ((flag_learn_mask==1) && (flag_share_lambda_parm==1)){
        grad_lambda/=T;//*sum(OnesMatV).T();
    }
    else{
        grad_lambda/=T;//*NVisible;
    }
    if (flag_learn_mask==1){
        gradThetaMask/=T;
    }

    Winc=momentum*Winc+learning_rate*(gradW*WMask);
    Wprime_inc=momentum*Wprime_inc+learning_rate*(gradWprime*WprimeMask);
    phi_inc=momentum*phi_inc+learning_rate*grad_phi*WMask;
    mu_inc=momentum*mu_inc+learning_rate*grad_mu;
    binc=momentum*binc+learning_rate*gradB;
    

    binit_inc=momentum*binit_inc+learning_rate*gradBinit;
    lambda_parm_inc=momentum*lambda_parm_inc+learning_rate*grad_lambda;

    if (flag_learn_mask==1){
        array Reg=(ThetaMask>kappa).as(f32)-(ThetaMask<kappa).as(f32);
        ThetaMask_inc=momentum*ThetaMask_inc+learning_rate*(gradThetaMask-l1_Theta_reg*Reg);
    }

    W+=Winc;
    Wprime+=Wprime_inc;
    b+=binc;
    binit+=binit_inc;
    phi+=phi_inc;
    phi(phi<0)=0;
    mu+=mu_inc;
    lambda_parm+=lambda_parm_inc;

    if (flag_learn_mask==1){
        ThetaMask+=ThetaMask_inc;
        WMask=Theta2MaskMat(ThetaMask, OnesMatV, OnesMatH);
        WprimeMask=Theta2MaskMat(ThetaMask, OnesMatH, OnesMatH);
    }
}

void ssRTRBM::load_data_to_GPU(std::vector<std::valarray<float> > data, std::valarray<int> T0){
    int Dlen=data.size();
    int i,j;
    std::vector<int> indices2;
    for (i=0;i<Dlen;i++){
        indices2.push_back(i);
    }
    std::random_shuffle( indices2.begin(), indices2.end());
    std::valarray<int> indices(0,Dlen);
    std::vector<int>::iterator jt;
    jt=indices2.begin();
    for (i=0;i<Dlen;i++){
        indices[i]=(*jt);
        jt++;
    }
    float *tmp_data = new float[NVisible*Dlen*((int)T0.max())];
    std::vector<std::valarray<float> >::iterator it;
    it = data.begin();

    for (i=0;i<Dlen;i++){
        for (j=0;j<NVisible*T0[indices[i]];j++){
            tmp_data[j+NVisible*((int)T0.max())*i]=(*(it+indices[i]))[j];
        }
    }

    data_GPU=array(NVisible,(int)T0.max(),Dlen,tmp_data);

    delete [] tmp_data;

    T.resize(T0.size());
    for (i=0;i<Dlen;i++){
        T[i]=T0[indices[i]];
    }
}

void ssRTRBM::load_data_to_GPU_without_reshuffling(std::vector<std::valarray<float> > data, std::valarray<int> T0){
    int Dlen=data.size();
    int i,j;

    float *tmp_data = new float[NVisible*Dlen*((int)T0.max())];
    std::vector<std::valarray<float> >::iterator it;
    it = data.begin();
    
    for (i=0;i<Dlen;i++){
        for (j=0;j<NVisible*T0[i];j++){
            tmp_data[j+NVisible*((int)T0.max())*i]=(*(it+i))[j];
        }
    }
    
    data_GPU=array(NVisible,(int)T0.max(),Dlen,tmp_data);
    
    T.resize(T0.size());
    T=T0;
    delete [] tmp_data;
}

void ssRTRBM::updateW(array W0){
    W=W0;
}

array ssRTRBM::getW(){
    return W;
}

array ssRTRBM::getWprime(){
    return Wprime;
}

array ssRTRBM::getb(){
    return b;
}

array ssRTRBM::getc(){
    return c;
}

array ssRTRBM::getPI(){
    return pi;
}

void ssRTRBM::set_learning_rate(float alpha0){
    learning_rate=constant((float)alpha0,1);
}


void ssRTRBM::update_params(){
    dim4 dims = data_GPU.dims();
    const int DLen=dims[2];
    for (int i = 0; i < DLen; i+=MiniBatchLen){
        update_batch(data_GPU(span,seq(T[i]),i));     
    }
}

void ssRTRBM::save_parameters(char* filename){
    std::ofstream outfile;
    int i,j;
    array WM=W*WMask;
    float* Wtmp = new float[NVisible*NHidden];
    WM.host(Wtmp);

    array WprimeM=Wprime*WprimeMask;
    float* WtmpPrime = new float[NHidden*NHidden];
    WprimeM.host(WtmpPrime);
    
    float* btmp = new float[NHidden];
    b.host(btmp);
    
    float* binittmp = new float[NHidden];
    binit.host(binittmp);

    array phiM=phi*WMask;
    float* phitmp = new float[NVisible*NHidden];
    phiM.host(phitmp);
    
    float* mutmp = new float[NHidden];
    mu.host(mutmp);

    outfile.open(filename);
    outfile << (double)NVisible << std::endl;
    outfile << (double)NHidden << std::endl;
    float alpha_parm0;
    float *lambda_parm0=new float[NVisible];
    alpha_parm.host(&alpha_parm0);
    lambda_parm.host(lambda_parm0);
    outfile << (double)alpha_parm0<< std::endl;
    array tmp_lambda_parm;
    if (lambda_parm.dims(0)>1){
        tmp_lambda_parm=matmul(OnesMatV,lambda_parm);
    }
    else{
        tmp_lambda_parm=tile(lambda_parm,NVisible,1);
    }
    tmp_lambda_parm.host(lambda_parm0);
    for (i=0;i<NVisible;i++){
        outfile << (double)lambda_parm0[i]<<std::endl;
    }
    
    outfile << (double)CD->get_flag_binary_real_ssRBM() << std::endl;

    for (i=0;i<NVisible;i++){
        for (j=0;j<NHidden;j++){
            outfile<<(double)Wtmp[i+NVisible*j] << std::endl;
        }
    }
    
    for (i=0;i<NHidden;i++){
        for (j=0;j<NHidden;j++){
            outfile<<(double)WtmpPrime[i+NHidden*j] << std::endl;
        }
    }
    for (j=0;j<NHidden;j++){
        outfile<<(double)btmp[j] << std::endl;
    }
    
    for (j=0;j<NHidden;j++){
        outfile<<(double)binittmp[j] << std::endl;
    }
    
    for (i=0;i<NVisible;i++){
        for (j=0;j<NHidden;j++){
            outfile<<(double)phitmp[i+NVisible*j] << std::endl;
        }
    }
    for (j=0;j<NHidden;j++){
        outfile<<(double)mutmp[j] << std::endl;
    }

    outfile<<(double)flag_learn_mask << std::endl;
    if (flag_learn_mask==1){
        outfile<<(double)num_of_nodes << std::endl;
        array Mat=Theta2Mat(ThetaMask);
        float* Masktmp = new float[num_of_nodes*num_of_nodes];
        Mat.host(Masktmp);
        for (j=0;j<num_of_nodes;j++){
            for (i=0;i<num_of_nodes;i++){
                outfile<<(double)Masktmp[i+num_of_nodes*j] << std::endl;
            }
        }
        delete [] Masktmp;
    }
    outfile.close();
    delete [] Wtmp;
    delete [] WtmpPrime;
    delete [] btmp;
    delete [] binittmp;
    delete [] phitmp;
    delete [] mutmp;
    delete [] lambda_parm0;
}


void ssRTRBM::read_parameters(char* filename){
    std::ifstream infile;
    double tmp_read;
    int i,j;
    infile.open(filename);
    infile >> tmp_read;
    NVisible=(int)tmp_read;
    infile >> tmp_read;
    NHidden=(int)tmp_read;
    infile >> tmp_read;
    alpha_parm=constant((float)tmp_read,1);
    float *lambda_parm0= new float[NVisible];
    for (i=0;i<NVisible;i++){
        infile >> tmp_read;
        lambda_parm0[i]=(float)tmp_read;
    }
    lambda_parm=array(NVisible,lambda_parm0);
    
    infile >> tmp_read;
    int flag_binary_real_ssRBM=(int)tmp_read;
    
    CD->set_flags(flag_binary_real_ssRBM);
    CD->set_NHidden(NHidden);
    
    float* Wtmp = new float[NVisible*NHidden];
    for (i=0;i<NVisible;i++){
        for (j=0;j<NHidden;j++){
            infile >> tmp_read;
            Wtmp[i+j*NVisible]=(float)tmp_read;
        }
    }
    W=array(NVisible,NHidden,Wtmp);
    
    float* WtmpPrime = new float[NHidden*NHidden];
    for (i=0;i<NHidden;i++){
        for (j=0;j<NHidden;j++){
            infile >> tmp_read;
            WtmpPrime[i+j*NHidden]=(float)tmp_read;
        }
    }
    Wprime=array(NHidden,NHidden,WtmpPrime);
    
    
    float* btmp = new float[NHidden];
    for (j=0;j<NHidden;j++){
        infile >> tmp_read;
        btmp[j]=(float)tmp_read;
    }
    b=array(NHidden,btmp);
    
    float* binittmp = new float[NHidden];
    for (j=0;j<NHidden;j++){
        infile >> tmp_read;
        binittmp[j]=(float)tmp_read;
    }
    binit=array(NHidden,binittmp);
    
    float* phitmp = new float[NVisible*NHidden];
    for (i=0;i<NVisible;i++){
        for (j=0;j<NHidden;j++){
            infile >> tmp_read;
            phitmp[i+NVisible*j]=(float)tmp_read;
        }
    }
    phi=array(NVisible,NHidden,phitmp);
    
    float* mutmp = new float[NHidden];
    for (j=0;j<NHidden;j++){
        infile >> tmp_read;
        mutmp[j]=(float)tmp_read;
    }
    mu=array(NHidden,mutmp);
    
    
    CD->updateParms(W,b,c,phi,mu,alpha_parm,lambda_parm);
    infile >> tmp_read;
    flag_learn_mask = (int)tmp_read;
    
    infile.close();
    delete [] Wtmp;
    delete [] WtmpPrime;
    delete [] btmp;
    delete [] binittmp;
    delete [] phitmp;
    delete [] mutmp;
    delete [] lambda_parm0;
}

array ssRTRBM::getMask(){
    return ThetaMask;
}

void ssRTRBM::setMasks(array Mat){
    WMask=matmul(matmul(OnesMatV,Mat),OnesMatH.T());
    WprimeMask=matmul(matmul(OnesMatH,Mat),OnesMatH.T());
}

float ssRTRBM::PredictionError2(std::valarray<float> Vdata, int T0){
    float *tmp_data = new float[NVisible*T0];
    int i, t;
    for (i=0;i<NVisible;i++){
        for (t=0;t<T0;t++){
            tmp_data[i+NVisible*t]=Vdata[i+NVisible*t];
        }
    }
    array generatedV=array(NVisible,T0,tmp_data);
    delete [] tmp_data;
    array tm0=matmul(generatedV(span,0).T(),W).T();
    array tmp=0.5/alpha_parm*tm0*tm0+matmul(generatedV(span,0).T(),W).T()*mu-0.5*matmul(generatedV(span,0).T()*generatedV(span,0).T(),phi).T()+binit;
    array rtmp=1/(exp(-tmp)+1);
    array htmp=matmul(Wprime,rtmp);//binit-b;
    array tmp_sum=constant(0,1);
    array tmp_arr;
    for (t=1;t<T0;t++){
        CD->updateParms(W,htmp+b,c,phi,mu, alpha_parm, lambda_parm);
        CD->sampleCD(generatedV(span,t-1).T());
        tmp_arr=CD->get_Ev_b();
        tmp_sum+=sum((generatedV(span,t)-tmp_arr.T())*(generatedV(span,t)-tmp_arr.T()));
        tm0=matmul(generatedV(span,t).T(),W).T();
        tmp=0.5/alpha_parm*tm0*tm0+matmul(generatedV(span,t).T(),W).T()*mu-0.5*matmul(generatedV(span,t).T()*generatedV(span,t).T(),phi).T()+b+matmul(Wprime,rtmp);
        rtmp=1/(exp(-tmp)+1);
        htmp=matmul(Wprime,rtmp);
    }
    float host_sum;
    tmp_sum.host(&host_sum);
    
    return (float)host_sum/(float)(T0-1);
}

float ssRTRBM::PredictionError2Half(std::valarray<float> Vdata, int T0, std::valarray<int> index){
    array indices_tmp=constant(0,NVisible,1);
    for (int i=0;i<(int)index.size();i++){
        indices_tmp(index[i])=1;
    }
    array indices=where(indices_tmp==1);
    array indices0=where(indices_tmp==0);
    float *tmp_data = new float[NVisible*T0];
    int i, t;
    for (i=0;i<NVisible;i++){
        for (t=0;t<T0;t++){
            tmp_data[i+NVisible*t]=Vdata[i+NVisible*t];
        }
    }
    array generatedV=array(NVisible,T0,tmp_data);
    delete [] tmp_data;
    array tm0=matmul(generatedV(span,0).T(),W).T();
    array tmp=0.5/alpha_parm*tm0*tm0+matmul(generatedV(span,0).T(),W).T()*mu-0.5*matmul(generatedV(span,0).T()*generatedV(span,0).T(),phi).T()+binit;
    array rtmp=1/(exp(-tmp)+1);
    array htmp=matmul(Wprime,rtmp);//binit-b;
    
    
    array tmp_arr;
    for (t=1;t<T0/2;t++){
        CD->updateParms(W,htmp+b,c,phi,mu, alpha_parm, lambda_parm);
        CD->sampleCD(generatedV(span,t-1).T());
        tmp_arr=CD->get_Ev_b();

        tm0=matmul(generatedV(span,t).T(),W).T();
        tmp=0.5/alpha_parm*tm0*tm0+matmul(generatedV(span,t).T(),W).T()*mu-0.5*matmul(generatedV(span,t).T()*generatedV(span,t).T(),phi).T()+b+matmul(Wprime,rtmp);
        rtmp=1/(exp(-tmp)+1);
        htmp=matmul(Wprime,rtmp);
    }
    
    array tmp_sum=constant(0,1);
    for (t=T0/2;t<T0;t++){
        CD->updateParms(W,htmp+b,c,phi,mu, alpha_parm, lambda_parm);
        array tmpv0=generatedV(span,t-1);
        tmpv0(indices0)=generatedV(span,t)(indices0);
        CD->MeanFieldV(tmpv0.T(),generatedV(span,t).T().T(), indices0);
        tmp_arr=CD->get_Ev_b();
        
        tmp_sum+=sum((generatedV(indices,t)-tmp_arr.T()(indices))*(generatedV(indices,t)-tmp_arr.T()(indices)));
        array VH=generatedV(span,t);
        VH(indices)=CD->get_Ev_b()(indices);
        generatedV(span,t)=VH;
        tm0=matmul(VH.T(),W).T();
        tmp=0.5/alpha_parm*tm0*tm0+matmul(VH.T(),W).T()*mu-0.5*matmul(VH.T()*VH.T(),phi).T()+b+matmul(Wprime,rtmp);
        rtmp=1/(exp(-tmp)+1);
        htmp=matmul(Wprime,rtmp);
    }
    
    float host_sum;
    tmp_sum.host(&host_sum);
    
    return (float)host_sum/(float)(T0-T0/2);
}

ssRTRBM::~ssRTRBM(){
}
